@@ -39,10 +39,12 @@ use crate::{
     OPT_NUMERIC_SUFFIXES_SHORT, OPT_SUFFIX_LENGTH,
 };
 use clap::ArgMatches;
-use std::fmt;
+use std::ffi::{OsStr, OsString};
 use std::path::is_separator;
+use thiserror::Error;
 use uucore::display::Quotable;
 use uucore::error::{UResult, USimpleError};
+use uucore::translate;
 
 /// The format to use for suffixes in the filename for each output chunk.
 #[derive(Clone, Copy)]
@@ -75,35 +77,25 @@ pub struct Suffix {
     length: usize,
     start: usize,
     auto_widening: bool,
-    additional: String,
+    additional: OsString,
 }
 
 /// An error when parsing suffix parameters from command-line arguments.
+#[derive(Debug, Error)]
 pub enum SuffixError {
     /// Invalid suffix length parameter.
+    #[error("{}", translate!("split-error-suffix-not-parsable", "value" => .0.quote()))]
     NotParsable(String),
 
     /// Suffix contains a directory separator, which is not allowed.
+    #[error("{}", translate!("split-error-suffix-contains-separator", "value" => .0.quote()))]
     ContainsSeparator(String),
 
     /// Suffix is not large enough to split into specified chunks
+    #[error("{}", translate!("split-error-suffix-too-small", "length" => .0))]
     TooSmall(usize),
 }
 
-impl fmt::Display for SuffixError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            Self::NotParsable(s) => write!(f, "invalid suffix length: {}", s.quote()),
-            Self::TooSmall(i) => write!(f, "the suffix length needs to be at least {i}"),
-            Self::ContainsSeparator(s) => write!(
-                f,
-                "invalid suffix {}, contains directory separator",
-                s.quote()
-            ),
-        }
-    }
-}
-
 impl Suffix {
     /// Parse the suffix type, start, length and additional suffix from the command-line arguments
     /// as well process suffix length auto-widening and auto-width scenarios
@@ -200,7 +192,7 @@ impl Suffix {
         }
 
         // Auto pre-calculate new suffix length (auto-width) if necessary
-        if let Strategy::Number(ref number_type) = strategy {
+        if let Strategy::Number(number_type) = strategy {
             let chunks = number_type.num_chunks();
             let required_length = ((start as u64 + chunks) as f64)
                 .log(stype.radix() as f64)
@@ -228,11 +220,13 @@ impl Suffix {
         }
 
         let additional = matches
-            .get_one::<String>(OPT_ADDITIONAL_SUFFIX)
+            .get_one::<OsString>(OPT_ADDITIONAL_SUFFIX)
             .unwrap()
-            .to_string();
-        if additional.chars().any(is_separator) {
-            return Err(SuffixError::ContainsSeparator(additional));
+            .clone();
+        if additional.to_string_lossy().chars().any(is_separator) {
+            return Err(SuffixError::ContainsSeparator(
+                additional.to_string_lossy().to_string(),
+            ));
         }
 
         let result = Self {
@@ -309,14 +303,14 @@ impl Suffix {
 /// assert_eq!(it.next().unwrap(), "chunk_02.txt");
 /// ```
 pub struct FilenameIterator<'a> {
-    prefix: &'a str,
-    additional_suffix: &'a str,
+    prefix: &'a OsStr,
+    additional_suffix: &'a OsStr,
     number: Number,
     first_iteration: bool,
 }
 
 impl<'a> FilenameIterator<'a> {
-    pub fn new(prefix: &'a str, suffix: &'a Suffix) -> UResult<Self> {
+    pub fn new(prefix: &'a OsStr, suffix: &'a Suffix) -> UResult<Self> {
         let radix = suffix.stype.radix();
         let number = if suffix.auto_widening {
             Number::DynamicWidth(DynamicWidthNumber::new(radix, suffix.start))
@@ -325,12 +319,12 @@ impl<'a> FilenameIterator<'a> {
                 FixedWidthNumber::new(radix, suffix.length, suffix.start).map_err(|_| {
                     USimpleError::new(
                         1,
-                        "numerical suffix start value is too large for the suffix length",
+                        translate!("split-error-numerical-suffix-start-too-large"),
                     )
                 })?,
             )
         };
-        let additional_suffix = suffix.additional.as_str();
+        let additional_suffix = &suffix.additional;
 
         Ok(FilenameIterator {
             prefix,
@@ -341,7 +335,7 @@ impl<'a> FilenameIterator<'a> {
     }
 }
 
-impl<'a> Iterator for FilenameIterator<'a> {
+impl Iterator for FilenameIterator<'_> {
     type Item = String;
 
     fn next(&mut self) -> Option<Self::Item> {
@@ -354,7 +348,9 @@ impl<'a> Iterator for FilenameIterator<'a> {
         // struct parameters unchanged.
         Some(format!(
             "{}{}{}",
-            self.prefix, self.number, self.additional_suffix
+            self.prefix.to_string_lossy(),
+            self.number,
+            self.additional_suffix.to_string_lossy()
         ))
     }
 }
@@ -373,14 +369,14 @@ mod tests {
             length: 2,
             start: 0,
             auto_widening: false,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_aa.txt");
         assert_eq!(it.next().unwrap(), "chunk_ab.txt");
         assert_eq!(it.next().unwrap(), "chunk_ac.txt");
 
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.nth(26 * 26 - 1).unwrap(), "chunk_zz.txt");
         assert_eq!(it.next(), None);
     }
@@ -392,14 +388,14 @@ mod tests {
             length: 2,
             start: 0,
             auto_widening: false,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_00.txt");
         assert_eq!(it.next().unwrap(), "chunk_01.txt");
         assert_eq!(it.next().unwrap(), "chunk_02.txt");
 
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.nth(10 * 10 - 1).unwrap(), "chunk_99.txt");
         assert_eq!(it.next(), None);
     }
@@ -411,14 +407,14 @@ mod tests {
             length: 2,
             start: 0,
             auto_widening: true,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_aa.txt");
         assert_eq!(it.next().unwrap(), "chunk_ab.txt");
         assert_eq!(it.next().unwrap(), "chunk_ac.txt");
 
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.nth(26 * 25 - 1).unwrap(), "chunk_yz.txt");
         assert_eq!(it.next().unwrap(), "chunk_zaaa.txt");
         assert_eq!(it.next().unwrap(), "chunk_zaab.txt");
@@ -431,14 +427,14 @@ mod tests {
             length: 2,
             start: 0,
             auto_widening: true,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_00.txt");
         assert_eq!(it.next().unwrap(), "chunk_01.txt");
         assert_eq!(it.next().unwrap(), "chunk_02.txt");
 
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.nth(10 * 9 - 1).unwrap(), "chunk_89.txt");
         assert_eq!(it.next().unwrap(), "chunk_9000.txt");
         assert_eq!(it.next().unwrap(), "chunk_9001.txt");
@@ -451,9 +447,9 @@ mod tests {
             length: 2,
             start: 5,
             auto_widening: true,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_05.txt");
         assert_eq!(it.next().unwrap(), "chunk_06.txt");
         assert_eq!(it.next().unwrap(), "chunk_07.txt");
@@ -466,9 +462,9 @@ mod tests {
             length: 2,
             start: 9,
             auto_widening: true,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_09.txt");
         assert_eq!(it.next().unwrap(), "chunk_0a.txt");
         assert_eq!(it.next().unwrap(), "chunk_0b.txt");
@@ -481,9 +477,9 @@ mod tests {
             length: 3,
             start: 999,
             auto_widening: false,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_999.txt");
         assert!(it.next().is_none());
 
@@ -492,9 +488,9 @@ mod tests {
             length: 3,
             start: 1000,
             auto_widening: false,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let it = FilenameIterator::new("chunk_", &suffix);
+        let it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix);
         assert!(it.is_err());
 
         let suffix = Suffix {
@@ -502,9 +498,9 @@ mod tests {
             length: 3,
             start: 0xfff,
             auto_widening: false,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let mut it = FilenameIterator::new("chunk_", &suffix).unwrap();
+        let mut it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix).unwrap();
         assert_eq!(it.next().unwrap(), "chunk_fff.txt");
         assert!(it.next().is_none());
 
@@ -513,9 +509,9 @@ mod tests {
             length: 3,
             start: 0x1000,
             auto_widening: false,
-            additional: ".txt".to_string(),
+            additional: ".txt".into(),
         };
-        let it = FilenameIterator::new("chunk_", &suffix);
+        let it = FilenameIterator::new(std::ffi::OsStr::new("chunk_"), &suffix);
         assert!(it.is_err());
     }
 }
