@@ -10,8 +10,6 @@ use std::time::Duration;
 
 use uutests::at_and_ucmd;
 use uutests::new_ucmd;
-use uutests::util::TestScenario;
-use uutests::util_name;
 
 fn test_helper(file_name: &str, possible_args: &[&str]) {
     for args in possible_args {
@@ -38,8 +36,7 @@ fn test_buffer_sizes() {
     #[cfg(not(target_os = "linux"))]
     let buffer_sizes = ["0", "50K", "50k", "1M", "100M"];
     for buffer_size in &buffer_sizes {
-        TestScenario::new(util_name!())
-            .ucmd()
+        new_ucmd!()
             .arg("-n")
             .arg("-S")
             .arg(buffer_size)
@@ -52,8 +49,7 @@ fn test_buffer_sizes() {
     {
         let buffer_sizes = ["1000G", "10T"];
         for buffer_size in &buffer_sizes {
-            TestScenario::new(util_name!())
-                .ucmd()
+            new_ucmd!()
                 .arg("-n")
                 .arg("-S")
                 .arg(buffer_size)
@@ -1007,8 +1003,7 @@ fn test_compress_merge() {
 #[cfg(not(target_os = "android"))]
 fn test_compress_fail() {
     #[cfg(not(windows))]
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .args(&[
             "ext_sort.txt",
             "-n",
@@ -1023,8 +1018,7 @@ fn test_compress_fail() {
     // "thread 'main' panicked at 'called `Option::unwrap()` on ...
     // So, don't check the output
     #[cfg(windows)]
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .args(&[
             "ext_sort.txt",
             "-n",
@@ -1038,8 +1032,7 @@ fn test_compress_fail() {
 
 #[test]
 fn test_merge_batches() {
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .timeout(Duration::from_secs(120))
         .args(&["ext_sort.txt", "-n", "-S", "150b"])
         .succeeds()
@@ -1048,27 +1041,31 @@ fn test_merge_batches() {
 
 #[test]
 fn test_batch_size_invalid() {
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .arg("--batch-size=0")
         .fails_with_code(2)
         .stderr_contains("sort: invalid --batch-size argument '0'")
         .stderr_contains("sort: minimum --batch-size argument is '2'");
+
+    // with -m, the error path is a bit different
+    new_ucmd!()
+        .args(&["-m", "--batch-size=a"])
+        .fails_with_code(2)
+        .stderr_contains("sort: invalid --batch-size argument 'a'");
 }
 
 #[test]
 fn test_batch_size_too_large() {
     let large_batch_size = "18446744073709551616";
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .arg(format!("--batch-size={large_batch_size}"))
         .fails_with_code(2)
         .stderr_contains(format!(
             "--batch-size argument '{large_batch_size}' too large"
         ));
+
     #[cfg(target_os = "linux")]
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .arg(format!("--batch-size={large_batch_size}"))
         .fails_with_code(2)
         .stderr_contains("maximum --batch-size argument with current rlimit is");
@@ -1076,8 +1073,7 @@ fn test_batch_size_too_large() {
 
 #[test]
 fn test_merge_batch_size() {
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .arg("--batch-size=2")
         .arg("-m")
         .arg("--unique")
@@ -1102,8 +1098,7 @@ fn test_merge_batch_size_with_limit() {
     // 2 descriptors for CTRL+C handling logic (to be reworked at some point)
     // 2 descriptors for the input files (i.e. batch-size of 2).
     let limit_fd = 3 + 2 + 2;
-    TestScenario::new(util_name!())
-        .ucmd()
+    new_ucmd!()
         .limit(Resource::NOFILE, limit_fd, limit_fd)
         .arg("--batch-size=2")
         .arg("-m")
@@ -1204,13 +1199,12 @@ fn test_separator_null() {
 #[test]
 fn test_output_is_input() {
     let input = "a\nb\nc\n";
-    let scene = TestScenario::new(util_name!());
-    let at = &scene.fixtures;
+    let (at, mut ucmd) = at_and_ucmd!();
+
     at.touch("file");
     at.append("file", input);
-    scene
-        .ucmd()
-        .args(&["-m", "-u", "-o", "file", "file", "file", "file"])
+
+    ucmd.args(&["-m", "-u", "-o", "file", "file", "file", "file"])
         .succeeds();
     assert_eq!(at.read("file"), input);
 }
@@ -1345,3 +1339,569 @@ fn test_failed_write_is_reported() {
         .fails()
         .stderr_is("sort: write failed: 'standard output': No space left on device\n");
 }
+
+#[test]
+// Test for GNU tests/sort/sort.pl "o2"
+fn test_multiple_output_files() {
+    new_ucmd!()
+        .args(&["-o", "foo", "-o", "bar"])
+        .fails_with_code(2)
+        .stderr_is("sort: multiple output files specified\n");
+}
+
+#[test]
+fn test_output_file_with_leading_dash() {
+    let test_cases = [
+        (
+            ["--output", "--dash-file"],
+            "banana\napple\ncherry\n",
+            "apple\nbanana\ncherry\n",
+        ),
+        (
+            ["-o", "--another-dash-file"],
+            "zebra\nxray\nyak\n",
+            "xray\nyak\nzebra\n",
+        ),
+    ];
+
+    for (args, input, expected) in test_cases {
+        let (at, mut ucmd) = at_and_ucmd!();
+        ucmd.args(&args).pipe_in(input).succeeds().no_stdout();
+
+        assert_eq!(at.read(args[1]), expected);
+    }
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "f-extra-arg"
+fn test_files0_from_extra_arg() {
+    new_ucmd!()
+        .args(&["--files0-from", "-", "foo"])
+        .fails_with_code(2)
+        .stderr_contains(
+            "sort: extra operand 'foo'\nfile operands cannot be combined with --files0-from\n",
+        )
+        .no_stdout();
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "missing"
+fn test_files0_from_missing() {
+    new_ucmd!()
+        .args(&["--files0-from", "missing_file"])
+        .fails_with_code(2)
+        .stderr_only(
+            #[cfg(not(windows))]
+            "sort: open failed: missing_file: No such file or directory\n",
+            #[cfg(windows)]
+            "sort: open failed: missing_file: The system cannot find the file specified.\n",
+        );
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "minus-in-stdin"
+fn test_files0_from_minus_in_stdin() {
+    new_ucmd!()
+        .args(&["--files0-from", "-"])
+        .pipe_in("-")
+        .fails_with_code(2)
+        .stderr_only("sort: when reading file names from stdin, no file name of '-' allowed\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "empty"
+fn test_files0_from_empty() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch("file");
+
+    ucmd.args(&["--files0-from", "file"])
+        .fails_with_code(2)
+        .stderr_only("sort: no input from 'file'\n");
+}
+
+#[cfg(target_os = "linux")]
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "empty-non-regular"
+fn test_files0_from_empty_non_regular() {
+    new_ucmd!()
+        .args(&["--files0-from", "/dev/null"])
+        .fails_with_code(2)
+        .stderr_only("sort: no input from '/dev/null'\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "nul-1"
+fn test_files0_from_nul() {
+    new_ucmd!()
+        .args(&["--files0-from", "-"])
+        .pipe_in("\0")
+        .fails_with_code(2)
+        .stderr_only("sort: -:1: invalid zero-length file name\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "nul-2"
+fn test_files0_from_nul2() {
+    new_ucmd!()
+        .args(&["--files0-from", "-"])
+        .pipe_in("\0\0")
+        .fails_with_code(2)
+        .stderr_only("sort: -:1: invalid zero-length file name\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "1"
+fn test_files0_from_1() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch("file");
+    at.append("file", "a");
+
+    ucmd.args(&["--files0-from", "-"])
+        .pipe_in("file")
+        .succeeds()
+        .stdout_only("a\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "1a"
+fn test_files0_from_1a() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch("file");
+    at.append("file", "a");
+
+    ucmd.args(&["--files0-from", "-"])
+        .pipe_in("file\0")
+        .succeeds()
+        .stdout_only("a\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "2"
+fn test_files0_from_2() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch("file");
+    at.append("file", "a");
+
+    ucmd.args(&["--files0-from", "-"])
+        .pipe_in("file\0file")
+        .succeeds()
+        .stdout_only("a\na\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "2a"
+fn test_files0_from_2a() {
+    let (at, mut ucmd) = at_and_ucmd!();
+
+    at.touch("file");
+    at.append("file", "a");
+
+    ucmd.args(&["--files0-from", "-"])
+        .pipe_in("file\0file\0")
+        .succeeds()
+        .stdout_only("a\na\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-files0-from.pl "zero-len"
+fn test_files0_from_zero_length() {
+    new_ucmd!()
+        .args(&["--files0-from", "-"])
+        .pipe_in("g\0\0b\0\0")
+        .fails_with_code(2)
+        .stderr_only("sort: -:2: invalid zero-length file name\n");
+}
+
+#[test]
+// Test for GNU tests/sort/sort-float.sh
+fn test_g_float() {
+    let input = "0\n-3.3621031431120935063e-4932\n3.3621031431120935063e-4932\n";
+    let output = "-3.3621031431120935063e-4932\n0\n3.3621031431120935063e-4932\n";
+    new_ucmd!()
+        .args(&["-g"])
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is(output);
+}
+
+#[test]
+// Test misc numbers ("'a" is not interpreted as literal, trailing text is ignored...)
+fn test_g_misc() {
+    let input = "1\n100\n90\n'a\n85hello\n";
+    let output = "'a\n1\n85hello\n90\n100\n";
+    new_ucmd!()
+        .args(&["-g"])
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is(output);
+}
+
+#[test]
+// Test numbers with a large number of digits, where only the last digit is different.
+// We use scientific notation to make sure string sorting does not correctly order them.
+fn test_g_arbitrary() {
+    let input = [
+        // GNU coreutils doesn't handle those correctly as they don't fit exactly in long double
+        "3",
+        "3.000000000000000000000000000000000000000000000000000000000000000004",
+        "0.3000000000000000000000000000000000000000000000000000000000000000002e1",
+        "0.03000000000000000000000000000000000000000000000000000000000000000003e2",
+        "0.003000000000000000000000000000000000000000000000000000000000000000001e3",
+        // GNU coreutils does handle those correctly though
+        "10",
+        "10.000000000000004",
+        "1.0000000000000002e1",
+        "0.10000000000000003e2",
+        "0.010000000000000001e3",
+    ]
+    .join("\n");
+    let output = [
+        "3",
+        "0.003000000000000000000000000000000000000000000000000000000000000000001e3",
+        "0.3000000000000000000000000000000000000000000000000000000000000000002e1",
+        "0.03000000000000000000000000000000000000000000000000000000000000000003e2",
+        "3.000000000000000000000000000000000000000000000000000000000000000004",
+        "10",
+        "0.010000000000000001e3",
+        "1.0000000000000002e1",
+        "0.10000000000000003e2",
+        "10.000000000000004",
+    ]
+    .join("\n")
+        + "\n";
+    new_ucmd!()
+        .args(&["-g"])
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is(output);
+}
+
+#[test]
+// Test hexadecimal numbers (and hex floats)
+fn test_g_float_hex() {
+    let input = "0x123\n0x0\n0x2p10\n0x9p-10\n";
+    let output = "0x0\n0x9p-10\n0x123\n0x2p10\n";
+    new_ucmd!()
+        .args(&["-g"])
+        .pipe_in(input)
+        .succeeds()
+        .stdout_is(output);
+}
+
+/* spell-checker: disable */
+#[test]
+fn test_french_translations() {
+    // Test that French translations work for clap error messages
+    // Set LANG to French and test with an invalid argument
+    let result = new_ucmd!()
+        .env("LANG", "fr_FR.UTF-8")
+        .env("LC_ALL", "fr_FR.UTF-8")
+        .arg("--invalid-arg")
+        .fails();
+
+    let stderr = result.stderr_str();
+    assert!(stderr.contains("erreur"));
+    assert!(stderr.contains("argument inattendu"));
+    assert!(stderr.contains("trouvé"));
+}
+
+#[test]
+fn test_argument_suggestion() {
+    let test_cases = vec![
+        ("en_US.UTF-8", vec!["tip", "similar", "--reverse"]),
+        ("fr_FR.UTF-8", vec!["conseil", "similaire", "--reverse"]),
+    ];
+
+    for (locale, expected_strings) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .arg("--revrse") // Typo
+            .fails();
+
+        let stderr = result.stderr_str();
+        for expected in expected_strings {
+            assert!(stderr.contains(expected));
+        }
+    }
+}
+
+#[test]
+fn test_clap_localization_unknown_argument() {
+    let test_cases = vec![
+        (
+            "en_US.UTF-8",
+            vec![
+                "error: unexpected argument '--unknown-option' found",
+                "Usage:",
+                "For more information, try '--help'.",
+            ],
+        ),
+        (
+            "fr_FR.UTF-8",
+            vec![
+                "erreur : argument inattendu '--unknown-option' trouvé",
+                "Utilisation:",
+                "Pour plus d'informations, essayez '--help'.",
+            ],
+        ),
+    ];
+
+    for (locale, expected_strings) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .arg("--unknown-option")
+            .fails();
+
+        result.code_is(2); // sort uses exit code 2 for invalid options
+        let stderr = result.stderr_str();
+        for expected in expected_strings {
+            assert!(stderr.contains(expected));
+        }
+    }
+}
+
+#[test]
+fn test_clap_localization_help_message() {
+    // Test help message in English
+    let result_en = new_ucmd!()
+        .env("LANG", "en_US.UTF-8")
+        .env("LC_ALL", "en_US.UTF-8")
+        .arg("--help")
+        .succeeds();
+
+    let stdout_en = result_en.stdout_str();
+    assert!(stdout_en.contains("Usage:"));
+    assert!(stdout_en.contains("Options:"));
+
+    // Test help message in French
+    let result_fr = new_ucmd!()
+        .env("LANG", "fr_FR.UTF-8")
+        .env("LC_ALL", "fr_FR.UTF-8")
+        .arg("--help")
+        .succeeds();
+
+    let stdout_fr = result_fr.stdout_str();
+    assert!(stdout_fr.contains("Utilisation:"));
+    assert!(stdout_fr.contains("Options:"));
+}
+
+#[test]
+fn test_clap_localization_missing_required_argument() {
+    // Test missing required argument
+    let result_en = new_ucmd!().env("LC_ALL", "en_US.UTF-8").arg("-k").fails();
+
+    let stderr_en = result_en.stderr_str();
+    assert!(stderr_en.contains(" a value is required for '--key <key>' but none was supplied"));
+    assert!(stderr_en.contains("-k"));
+}
+
+#[test]
+fn test_clap_localization_invalid_value() {
+    let test_cases = vec![
+        ("en_US.UTF-8", "sort: failed to parse key 'invalid'"),
+        ("fr_FR.UTF-8", "sort: échec d'analyse de la clé 'invalid'"),
+    ];
+
+    for (locale, expected_message) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .arg("-k")
+            .arg("invalid")
+            .fails();
+
+        let stderr = result.stderr_str();
+        assert!(stderr.contains(expected_message));
+    }
+}
+
+#[test]
+fn test_help_colors_enabled() {
+    // Test that help messages have ANSI color codes when colors are forced
+    let test_cases = vec![("en_US.UTF-8", "Usage"), ("fr_FR.UTF-8", "Utilisation")];
+
+    for (locale, usage_word) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .env("CLICOLOR_FORCE", "1")
+            .arg("--help")
+            .succeeds();
+
+        let stdout = result.stdout_str();
+
+        // Check for ANSI bold+underline codes around the usage header
+        let expected_pattern = format!("\x1b[1m\x1b[4m{usage_word}:\x1b[0m");
+        assert!(
+            stdout.contains(&expected_pattern),
+            "Expected bold+underline '{usage_word}:' in locale {locale}, got: {}",
+            stdout.lines().take(10).collect::<Vec<_>>().join("\\n")
+        );
+    }
+}
+
+#[test]
+fn test_help_colors_disabled() {
+    // Test that help messages don't have ANSI color codes when colors are disabled
+    let test_cases = vec![("en_US.UTF-8", "Usage"), ("fr_FR.UTF-8", "Utilisation")];
+
+    for (locale, usage_word) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .env("NO_COLOR", "1")
+            .arg("--help")
+            .succeeds();
+
+        let stdout = result.stdout_str();
+
+        // Check that we have the usage word but no ANSI codes
+        assert!(stdout.contains(&format!("{usage_word}:")));
+        assert!(
+            !stdout.contains("\x1b["),
+            "Found ANSI escape codes when colors should be disabled in locale {locale}"
+        );
+    }
+}
+
+#[test]
+fn test_error_colors_enabled() {
+    // Test that error messages have ANSI color codes when colors are forced
+    let test_cases = vec![
+        ("en_US.UTF-8", "error", "tip"),
+        ("fr_FR.UTF-8", "erreur", "conseil"),
+    ];
+
+    for (locale, error_word, tip_word) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .env("CLICOLOR_FORCE", "1")
+            .arg("--numerc") // Typo to trigger suggestion for --numeric-sort
+            .fails();
+
+        let stderr = result.stderr_str();
+
+        // Check for colored error word (red)
+        let colored_error = format!("\x1b[31m{error_word}\x1b[0m");
+        assert!(
+            stderr.contains(&colored_error),
+            "Expected red '{error_word}' in locale {locale}, got: {}",
+            stderr.lines().take(5).collect::<Vec<_>>().join("\\n")
+        );
+
+        // Check for colored tip word (green)
+        let colored_tip = format!("\x1b[32m{tip_word}\x1b[0m");
+        assert!(
+            stderr.contains(&colored_tip),
+            "Expected green '{tip_word}' in locale {locale}, got: {}",
+            stderr.lines().take(5).collect::<Vec<_>>().join("\\n")
+        );
+    }
+}
+
+#[test]
+fn test_error_colors_disabled() {
+    // Test that error messages don't have ANSI color codes when colors are disabled
+    let test_cases = vec![
+        ("en_US.UTF-8", "error", "tip"),
+        ("fr_FR.UTF-8", "erreur", "conseil"),
+    ];
+
+    for (locale, error_word, tip_word) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .env("NO_COLOR", "1")
+            .arg("--numerc") // Typo to trigger suggestion for --numeric-sort
+            .fails();
+
+        let stderr = result.stderr_str();
+
+        // Check that we have the error and tip words but no ANSI codes
+        assert!(stderr.contains(error_word));
+        assert!(stderr.contains(tip_word));
+        assert!(
+            !stderr.contains("\x1b["),
+            "Found ANSI escape codes when colors should be disabled in locale {locale}"
+        );
+    }
+}
+
+#[test]
+fn test_argument_suggestion_colors_enabled() {
+    // Test that argument suggestions have colors
+    let test_cases = vec![
+        ("en_US.UTF-8", "tip", "--reverse"),
+        ("fr_FR.UTF-8", "conseil", "--reverse"),
+    ];
+
+    for (locale, _tip_word, suggestion) in test_cases {
+        let result = new_ucmd!()
+            .env("LANG", locale)
+            .env("LC_ALL", locale)
+            .env("CLICOLOR_FORCE", "1")
+            .arg("--revrse") // Typo to trigger suggestion
+            .fails();
+
+        let stderr = result.stderr_str();
+
+        // Check for colored invalid argument (yellow)
+        let colored_invalid = "\x1b[33m--revrse\x1b[0m";
+        assert!(
+            stderr.contains(colored_invalid),
+            "Expected yellow '--revrse' in locale {locale}, got: {}",
+            stderr.lines().take(10).collect::<Vec<_>>().join("\\n")
+        );
+
+        // Check for colored suggestion (green)
+        let colored_suggestion = format!("\x1b[32m{suggestion}\x1b[0m");
+        assert!(
+            stderr.contains(&colored_suggestion),
+            "Expected green '{suggestion}' in locale {locale}, got: {}",
+            stderr.lines().take(10).collect::<Vec<_>>().join("\\n")
+        );
+    }
+}
+
+#[test]
+fn test_color_environment_variables() {
+    // Test different color environment variable combinations
+    let test_env_vars = vec![
+        // Colors should be enabled
+        (vec![("CLICOLOR_FORCE", "1")], true, "CLICOLOR_FORCE=1"),
+        // Colors should be disabled
+        (vec![("NO_COLOR", "1")], false, "NO_COLOR=1"),
+        (
+            vec![("NO_COLOR", "1"), ("CLICOLOR_FORCE", "1")],
+            false,
+            "NO_COLOR overrides CLICOLOR_FORCE",
+        ),
+    ];
+
+    for (env_vars, should_have_colors, description) in test_env_vars {
+        let mut cmd = new_ucmd!();
+        cmd.env("LANG", "en_US.UTF-8");
+
+        for (key, value) in env_vars {
+            cmd.env(key, value);
+        }
+
+        let result = cmd.arg("--help").succeeds();
+        let stdout = result.stdout_str();
+
+        let has_ansi_codes = stdout.contains("\x1b[");
+        assert_eq!(
+            has_ansi_codes, should_have_colors,
+            "Color test failed for {description}: expected colors={should_have_colors}, found ANSI codes={has_ansi_codes}"
+        );
+    }
+}
+
+/* spell-checker: enable */
